package com.example.swiperawesome

import android.annotation.SuppressLint
import android.content.Context
import android.util.Log
import android.view.GestureDetector
import android.view.GestureDetector.SimpleOnGestureListener
import android.view.MotionEvent
import android.view.View
import android.view.View.OnTouchListener

open class OnSwipeTouchListener(c: Context) : OnTouchListener {
    companion object {
        private const val SWIPE_THRESHOLD = 100
        private const val SWIPE_VELOCITY_THRESHOLD = 100
        private const val TAG = "OnSwipeTouchListener"
    }
    //----------------------------------------------------------------------------------------------
    private val gestureDetector: GestureDetector
    init {
        gestureDetector = GestureDetector(c, GestureListener())
    }
    //----------------------------------------------------------------------------------------------
    @SuppressLint("ClickableViewAccessibility")
    override fun onTouch(view: View, motionEvent: MotionEvent): Boolean {
        return gestureDetector.onTouchEvent(motionEvent)
    }
    //----------------------------------------------------------------------------------------------
    private inner class GestureListener : SimpleOnGestureListener() {
        override fun onDown(e: MotionEvent): Boolean {
            return true
        }
        // Determines the fling velocity and then fires the appropriate swipe event accordingly
        override fun onFling(e1: MotionEvent, e2: MotionEvent, velocityX: Float, velocityY: Float): Boolean {
            val result = false
            try {
                val diffY = e2.y - e1.y
                val diffX = e2.x - e1.x
                if (Math.abs(diffX) > Math.abs(diffY)) {
                    if (Math.abs(diffX) > SWIPE_THRESHOLD && Math.abs(velocityX) > SWIPE_VELOCITY_THRESHOLD) {
                        if (diffX > 0) {
                            onSwipeRight()
                        } else {
                            onSwipeLeft()
                        }
                    }
                } else {
                    if (Math.abs(diffY) > SWIPE_THRESHOLD && Math.abs(velocityY) > SWIPE_VELOCITY_THRESHOLD) {
                        if (diffY > 0) {
                            onSwipeDown()
                        } else {
                            onSwipeUp()
                        }
                    }
                }
            } catch (exception: Exception) {
                exception.printStackTrace()
            }
            return result
        }

        override fun onDoubleTap(e: MotionEvent): Boolean {
            Log.i(OnSwipeTouchListener.TAG, "onDoubleTouch")
            onDoubleTouch()
            return true
        }

        override fun onLongPress(e: MotionEvent) {
            Log.i(OnSwipeTouchListener.TAG, "onLongTouch")
            onLongTouch()
            super.onLongPress(e)
        }

        override fun onSingleTapConfirmed(e: MotionEvent): Boolean {
            Log.i(OnSwipeTouchListener.TAG, "onSingleTouch")
            onSingleTouch()
            return super.onSingleTapConfirmed(e)
        }
    }
    //----------------------------------------------------------------------------------------------
    open fun onSwipeRight() {}
    open fun onSwipeLeft() {}
    open fun onSwipeUp() {}
    open fun onSwipeDown() {}
    open fun onDoubleTouch() {}
    open fun onLongTouch() {}
    open fun onSingleTouch() {}
}